/*
 * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, you can obtain one at https://mozilla.org/MPL/2.0/.
 *
 * See the COPYRIGHT file distributed with this work for additional
 * information regarding copyright ownership.
 */


/*! \file isccfg/version.h */

#include <isc/platform.h>

LIBISCCFG_EXTERNAL_DATA extern const char cfg_version[];

LIBISCCFG_EXTERNAL_DATA extern const unsigned int cfg_libinterface;
LIBISCCFG_EXTERNAL_DATA extern const unsigned int cfg_librevision;
LIBISCCFG_EXTERNAL_DATA extern const unsigned int cfg_libage;
