/* Library to set needed capabilities for current process */
#ifndef _SETCAP_H_
#define _SETCAP_H_

#include <sys/types.h>


// Switch on CAP_SYS_RESOURCE capabality to switch off disk quotas checking
//
// Returns 0 if successful
// Returns -1 if error has occured
int enable_quota_capability();

// Switch off CAP_SYS_RESOURCE capabality to switch off disk quotas checking
//
// Returns 0 if successful
// Returns -1 if error has occured
int disable_quota_capability();

#endif
