/*
 * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, you can obtain one at https://mozilla.org/MPL/2.0/.
 *
 * See the COPYRIGHT file distributed with this work for additional
 * information regarding copyright ownership.
 */

#ifndef LWRES_INT_H
#define LWRES_INT_H 1

#include <inttypes.h>

/*! \file lwres/int.h */

typedef lwres_int8_t			int8_t;
typedef lwres_uint8_t			uint8_t;
typedef lwres_int16_t			int16_t;
typedef lwres_uint16_t			uint16_t;
typedef lwres_int32_t			int32_t;
typedef lwres_uint32_t			uint32_t;
typedef lwres_int64_t			int64_t;
typedef lwres_uint64_t			uint64_t;

#endif /* LWRES_INT_H */
