/*
 * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, you can obtain one at https://mozilla.org/MPL/2.0/.
 *
 * See the COPYRIGHT file distributed with this work for additional
 * information regarding copyright ownership.
 */

#pragma once

/*! \file isc/boolean.h */

#define isc_boolean_t bool
#define isc_boolean_false false
#define isc_boolean_true true

#define ISC_FALSE false
#define ISC_TRUE true
#define ISC_TF(x) (!!(x))
