/*
 * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, you can obtain one at https://mozilla.org/MPL/2.0/.
 *
 * See the COPYRIGHT file distributed with this work for additional
 * information regarding copyright ownership.
 */


/*! \file */

#include <irs/platform.h>

LIBIRS_EXTERNAL_DATA extern const char irs_version[];

LIBIRS_EXTERNAL_DATA extern const unsigned int irs_libinterface;
LIBIRS_EXTERNAL_DATA extern const unsigned int irs_librevision;
LIBIRS_EXTERNAL_DATA extern const unsigned int irs_libage;
